import os
import os.path
import logging
import glob

#------------------------------------------------------------------------------
#           CreateRegistryEntries
#  A one time installation process that configures the local python environment
#  registry. This is typically a YAML based registry. It is intended that this
#  function is invoked as part of the python installation process. It is invoked
#  after pip install sasktranif. 
#
#  The code only works properly on Windows with sasktran core components
#  already installed. Linux users currently have to manually edit the python YAML
#  registry file.
#
#------------------------------------------------------------------------------

def _createDefaultRegistryEntries( registry ):

    defaultgeoschem  = r'\\datastore\valhalla\root\data_products\G5NR_HERACLES'
    registry.setkey( 'Software/USask-ARG/Climatology/GEOSCHEM5/Storage/GeosChem5Dir', defaultgeoschem)
    registry.save()
    
#------------------------------------------------------------------------------
#           check_installation_called_from_sasktranif
#   If you have installation issues you can track the logger output by
#   executing the following commands before importing sasktran and then look at
#   file example.log
#
#   >>> import logging
#   >>> logging.basicConfig(filename='example.log',level=logging.DEBUG)
#   >>> import sasktran
#------------------------------------------------------------------------------

def check_installation_called_from_sasktranif( SKTRAN_IFCreateRegistryEntriesForDLL, registry ):

    dirname = os.path.dirname(__file__)
    firstimefile = os.path.join( dirname, 'sasktran_gcm_firsttime.sktran' )
    if os.path.exists(firstimefile):
        logging.info("sasktran_gcm, first time setup: Initiliazing Sasktran GEM MACH registry settings")
        _createDefaultRegistryEntries(registry)
        dllnames = glob.glob( dirname + os.sep + "_sasktran_gcm_internals*")           # Search that directory for the core components DLL or shareable object
        dllname = os.path.normpath(dllnames[0])                                             # We must find at least 1 (or fail)
        SKTRAN_IFCreateRegistryEntriesForDLL(dllname, dllname)                              # param[0] =  full path name of this DLL.
        os.unlink(firstimefile)
        logging.info("First Time Setup: Sasktran GEM MACH Components default initialization is complete.")

#------------------------------------------------------------------------------
#           GEM_MACH
#   We have to put a cheat in the system  as we cannot load sasktran_gcm.gem_mach_climatology
#   in the main  __init__ script as it creates a circular reference back to sasktran and fails. This
#   function looks like the class method GEM_MACH in gem_mach_climatology but it is really a function
#------------------------------------------------------------------------------

def GEM_MACH():
    from . gem_mach_climatology import GEM_MACH
    return GEM_MACH()


def GEOS_CHEM():
    from . geoschem_climatology import GEOS_CHEM
    return GEOS_CHEM()

from ._version import get_versions
__version__ = get_versions()['version']
del get_versions

