import numpy as np
import sasktran
from sasktran.exceptions import wrap_skif_functionfail
import arg_time

class GEM_MACH(sasktran.Climatology):
    """
    A climatology for the GEM-MACH 2.5 km regional model. The code works with the netcdf4 files generated by Chris
    McLindens group at Environment Canada.

    Examples
    --------
    >>> from sasktran_gcm import GEM_MACH
    >>> gem = GEM_MACH()
    >>> print(gem)
    SasktranIF Climatology: GEM-MACH
    Supported Species: ['SKCLIMATOLOGY_PRESSURE_PA', 'SKCLIMATOLOGY_AIRNUMBERDENSITY_CM3', 'SKCLIMATOLOGY_TEMPERATURE_K', 'SKCLIMATOLOGY_O2_CM3']

    """
    def __init__(self):
        super().__init__('GEM-MACH')
        self._cache_has_been_called = False

    #------------------------------------------------------------------------------
    #           __repr__
    #------------------------------------------------------------------------------

    def __repr__(self):

        representation = ('SasktranIF Climatology: {}\n'
                          'Supported Species: {}'.format(self._name, self.supported_species()))

        return representation

    #------------------------------------------------------------------------------
    #           supported_species
    #------------------------------------------------------------------------------

    def supported_species(self):
        return [ 'SKCLIMATOLOGY_NO2_CM3',
                 'SKCLIMATOLOGY_AIRNUMBERDENSITY_CM3',
                 'SKCLIMATOLOGY_TEMPERATURE_K',
                 'SKCLIMATOLOGY_SURFACE_GEOMETRIC_HEIGHT',
                 'SKCLIMATOLOGY_SURFACE_GEOPOTENTIAL_HEIGHT',
                 'SKCLIMATOLOGY_CO_CM3',
                 'SKCLIMATOLOGY_CH4_CM3',
                 'SKCLIMATOLOGY_NH3_CM3',
                 'SKCLIMATOLOGY_NO_CM3',
                 'SKCLIMATOLOGY_O3_CM3',
                 'SKCLIMATOLOGY_SO2_CM3',
                 'SKCLIMATOLOGY_PRESSURE_PA',
                 'SKCLIMATOLOGY_NO2_VMR',
                 'SKCLIMATOLOGY_CO_VMR',
                 'SKCLIMATOLOGY_CH4_VMR',
                 'SKCLIMATOLOGY_NH3_VMR',
                 'SKCLIMATOLOGY_NO_VMR',
                 'SKCLIMATOLOGY_O3_VMR',
                 'SKCLIMATOLOGY_SO2_VMR']


    #------------------------------------------------------------------------------
    #           update_cache
    #------------------------------------------------------------------------------
    @wrap_skif_functionfail
    def update_cache(self, latitude: float, longitude: float, utc: float)->bool:
        """
        Updates the GEM-MACH internal cache. Once this function is successfully called the cache will stay frozen in
        memory until the next call to update_cache. The GEM-MACH cache is currently updated if the UTC goes out of
        range of the current cache.

        The user must make all the necessary calls to `set_gem_filename` and `add_species` before calling `update_cache`
        as only the species tagged for loading  will be loaded into the cache.

        Parameters
        ----------
        latitude : float
            Latitude in degrees.
        longitude : float
            Longitude in degrees.
        mjd : float
            A scalar UTC that can be converted to Modified julian date. It can be a datetime, datetime64 or (mjd) number

        Returns
        -------
        bool
            True if cache is successfully loaded.
        """

        mjd = arg_time.ut_to_mjd(utc)
        location = [latitude, longitude, 0, mjd]
        ok = self._iskclimatology.UpdateCache(location)
        self._cache_has_been_called = ok
        return ok

    #------------------------------------------------------------------------------
    #           get_parameter
    #------------------------------------------------------------------------------

    @wrap_skif_functionfail
    def get_parameter(self, species: str, latitude: float, longitude: float, altitudes: np.ndarray, utc: float)->np.ndarray:
        """
        Get a profile from the climatology. The code will always update the internal cache unless the user has previously
        explicitly called update_cache. NOte that you must make all the necessary calls to  `set_gem_filename` and
        `add_species` before the first call to `get_parameter`. Requesting to load species after the first call to
        `update_cache` or  `get_parameter` is not supported.

        Parameters
        ----------
        species : str
            Species identifier of the species to query. Corresponds to :py:attr:`sasktran.Species.species`. The species
            must have been requested by a previous call to `load_species`
        latitude : float
            Latitude in degrees.
        longitude : float
            Longitude in degrees.
        altitudes : np.ndarray
            Profile altitude grid in meters.
        mjd : float
            A scalar UTC that can be converted to Modified julian date. It can be a datetime, datetime64 or (mjd) number

        Returns
        -------
        np.ndarray
            Climatology values at the specified altitudes.
        """

        mjd = arg_time.ut_to_mjd(utc)
        location = [latitude, longitude, 0, mjd]
        if (not self._cache_has_been_called): self._iskclimatology.UpdateCache(location)
        return self._iskclimatology.GetHeightProfile(species, location, np.asarray(altitudes))[1]

    #------------------------------------------------------------------------------
    #           set_filename
    #------------------------------------------------------------------------------
    @wrap_skif_functionfail
    def set_gem_filename(self, filename: str)->bool:
        """
        Sets the filename that will be used for the GEM-MACH climatology. We currently do not have any standard naming
        conventions for the GEM-MACH files so the user must provide the name of the file they wish to use. This function
        must be called before the first call to `update_cache` or `get_parameter` otherwise the climatology will throw an
        exception when it tries to load the internal cache.

        Parameters
        ----------
        filename : str
            The full filename of the GEM-MACH netcdf file to be used for the climatology.

        Returns
        -------
        bool
            Returns True if successful.
        """
        return self._iskclimatology.SetProperty('set_gem_filename',filename)

    #------------------------------------------------------------------------------
    #           add_species
    #------------------------------------------------------------------------------
    @wrap_skif_functionfail
    def add_species(self, speciesname):
        """
        Instructs the GEM-MACH climatology to load the requested variable when it next loads the internal cache. The
        function may be called multiple times to request multiple variables be loaded as long as all the calls are made before
        the first call to `update_cache` or `get_parameter`.

        Parameters
        ----------
        speciesname : str
            The name of any species supported by GEM-MACH. Complete list of species can be found in method `supportedspecies`
            of in the GEM-MACH documentation.

        Returns
        -------
        bool
            Returns True if successful.
        """

        return self._iskclimatology.SetProperty('add_species',speciesname)