import numpy as np
import sasktran
from sasktran.exceptions import wrap_skif_functionfail
import arg_time

class GEOS_CHEM(sasktran.Climatology):
    """
    A climatology for the GEOS-CHEM model. The code works with the netcdf4 files archived at the  NSCC data portal.

    Examples
    --------
    >>> from sasktran_gcm import GEOS_CHEM
    >>> geos = GEOS_CHEM()
    >>> print(geos)
    SasktranIF Climatology: GEOS-CHEM
    Supported Species: ['SKCLIMATOLOGY_AIRNUMBERDENSITY_CM3', 'SKCLIMATOLOGY_PRESSURE_PA', 'SKCLIMATOLOGY_TEMPERATURE_K', 'SKCLIMATOLOGY_EPV',
                        'SKCLIMATOLOGY_GEOMETRIC_HEIGHT', 'SKCLIMATOLOGY_SURFACE_GEOPOTENTIAL_HEIGHT', 'SKCLIMATOLOGY_SURFACE_PRESSURE_PA',
                        'SKCLIMATOLOGY_CLOUD_FRACTION', 'SKCLIMATOLOGY_QI_MMR', 'SKCLIMATOLOGY_QL_MMR', 'SKCLIMATOLOGY_QV', 'SKCLIMATOLOGY_RH',
                        'SKCLIMATOLOGY_AOA_DAYS', 'SKCLIMATOLOGY_DUST_0p7mu', 'SKCLIMATOLOGY_DUST_1p4mu', 'SKCLIMATOLOGY_DUST_2p4mu',
                        'SKCLIMATOLOGY_DUST_4p5mu', 'SKCLIMATOLOGY_BCPI', 'SKCLIMATOLOGY_BCPO', 'SKCLIMATOLOGY_SALA', 'SKCLIMATOLOGY_SALC',
                        'SKCLIMATOLOGY_OCPI', 'SKCLIMATOLOGY_OCPO', 'SKCLIMATOLOGY_BRO_CM3', 'SKCLIMATOLOGY_BRO_VMR', 'SKCLIMATOLOGY_C3H6O_CM3',
                        'SKCLIMATOLOGY_C3H6O_VMR', 'SKCLIMATOLOGY_CH3I_CM3', 'SKCLIMATOLOGY_CH3I_VMR', 'SKCLIMATOLOGY_C5H8_CM3',
                        'SKCLIMATOLOGY_C5H8_VMR', 'SKCLIMATOLOGY_CH2O_CM3', 'SKCLIMATOLOGY_CH2O_VMR', 'SKCLIMATOLOGY_CH4_CM3', 'SKCLIMATOLOGY_CH4_VMR',
                        'SKCLIMATOLOGY_CO_CM3', 'SKCLIMATOLOGY_CO_VMR', 'SKCLIMATOLOGY_HNO2_CM3', 'SKCLIMATOLOGY_HNO2_VMR', 'SKCLIMATOLOGY_HNO3_CM3',
                        'SKCLIMATOLOGY_HNO3_VMR', 'SKCLIMATOLOGY_NH3_CM3', 'SKCLIMATOLOGY_NH3_VMR', 'SKCLIMATOLOGY_NITS', 'SKCLIMATOLOGY_NO2_CM3',
                        'SKCLIMATOLOGY_NO2_VMR', 'SKCLIMATOLOGY_NO_CM3', 'SKCLIMATOLOGY_NO_VMR', 'SKCLIMATOLOGY_O3_CM3', 'SKCLIMATOLOGY_O3_VMR',
                        'SKCLIMATOLOGY_PAN_CM3', 'SKCLIMATOLOGY_PAN_VMR', 'SKCLIMATOLOGY_SO2_CM3', 'SKCLIMATOLOGY_SO2_VMR', 'SKCLIMATOLOGY_SO4_CM3']

    """
    def __init__(self):
        super().__init__('GEOS-CHEM')
        self._cache_has_been_called = False

    #------------------------------------------------------------------------------
    #           __repr__
    #------------------------------------------------------------------------------

    def __repr__(self):

        representation = ('SasktranIF Climatology: {}\n'
                          'Supported Species: {}'.format(self._name, self.supported_species()))

        return representation

    #------------------------------------------------------------------------------
    #           supported_species
    #------------------------------------------------------------------------------

    def supported_species(self):
        return [    'SKCLIMATOLOGY_AIRNUMBERDENSITY_CM3',
                    'SKCLIMATOLOGY_PRESSURE_PA',
                    'SKCLIMATOLOGY_TEMPERATURE_K',
                    'SKCLIMATOLOGY_EPV',
                    'SKCLIMATOLOGY_GEOMETRIC_HEIGHT',
                    'SKCLIMATOLOGY_SURFACE_GEOPOTENTIAL_HEIGHT',
                    'SKCLIMATOLOGY_SURFACE_GEOMETRIC_HEIGHT',
                    'SKCLIMATOLOGY_SURFACE_PRESSURE_PA',
                    'SKCLIMATOLOGY_CLOUD_FRACTION',
                    'SKCLIMATOLOGY_QI_MMR',
                    'SKCLIMATOLOGY_QL_MMR',
                    'SKCLIMATOLOGY_QV',
                    'SKCLIMATOLOGY_RH',
                    'SKCLIMATOLOGY_AOA_DAYS',
                    'SKCLIMATOLOGY_DUST_0p7mu',
                    'SKCLIMATOLOGY_DUST_1p4mu',
                    'SKCLIMATOLOGY_DUST_2p4mu',
                    'SKCLIMATOLOGY_DUST_4p5mu',
                    'SKCLIMATOLOGY_BCPI',
                    'SKCLIMATOLOGY_BCPO',
                    'SKCLIMATOLOGY_SALA',
                    'SKCLIMATOLOGY_SALC',
                    'SKCLIMATOLOGY_OCPI',
                    'SKCLIMATOLOGY_OCPO',
                    'SKCLIMATOLOGY_BRO_CM3',
                    'SKCLIMATOLOGY_BRO_VMR',
                    'SKCLIMATOLOGY_C3H6O_CM3',
                    'SKCLIMATOLOGY_C3H6O_VMR',
                    'SKCLIMATOLOGY_CH3I_CM3',
                    'SKCLIMATOLOGY_CH3I_VMR',
                    'SKCLIMATOLOGY_C5H8_CM3',
                    'SKCLIMATOLOGY_C5H8_VMR',
                    'SKCLIMATOLOGY_CH2O_CM3',
                    'SKCLIMATOLOGY_CH2O_VMR',
                    'SKCLIMATOLOGY_CH4_CM3',
                    'SKCLIMATOLOGY_CH4_VMR',
                    'SKCLIMATOLOGY_CO_CM3',
                    'SKCLIMATOLOGY_CO_VMR',
                    'SKCLIMATOLOGY_HNO2_CM3',
                    'SKCLIMATOLOGY_HNO2_VMR',
                    'SKCLIMATOLOGY_HNO3_CM3',
                    'SKCLIMATOLOGY_HNO3_VMR',
                    'SKCLIMATOLOGY_NH3_CM3',
                    'SKCLIMATOLOGY_NH3_VMR',
                    'SKCLIMATOLOGY_NITS',
                    'SKCLIMATOLOGY_NO2_CM3',
                    'SKCLIMATOLOGY_NO2_VMR',
                    'SKCLIMATOLOGY_NO_CM3',
                    'SKCLIMATOLOGY_NO_VMR',
                    'SKCLIMATOLOGY_O3_CM3',
                    'SKCLIMATOLOGY_O3_VMR',
                    'SKCLIMATOLOGY_PAN_CM3',
                    'SKCLIMATOLOGY_PAN_VMR',
                    'SKCLIMATOLOGY_SO2_CM3',
                    'SKCLIMATOLOGY_SO2_VMR',
                    'SKCLIMATOLOGY_SO4_CM3' ]


    #------------------------------------------------------------------------------
    #           update_cache
    #------------------------------------------------------------------------------
    @wrap_skif_functionfail
    def update_cache(self, latitude: float, longitude: float, utc: float)->bool:
        """
        Updates the GEOS-CHEM internal cache. Once this function is successfully called the cache will stay frozen in
        memory until the next call to update_cache. This can be useful to avoid constant swapping of the cache. The
        GEOS-CHEM cache is currently only updated if the UTC goes out of range of the current cache.

        The user must make all the necessary calls to `add_species` before calling `update_cache`
        as only the species tagged for loading  will be loaded into the cache.

        Parameters
        ----------
        latitude : float
            Latitude in degrees.
        longitude : float
            Longitude in degrees.
        mjd : float
            A scalar UTC that can be converted to Modified julian date. It can be a datetime, datetime64 or (mjd) number

        Returns
        -------
        bool
            True if cache is successfully loaded.
        """

        mjd = arg_time.ut_to_mjd(utc)
        location = [latitude, longitude, 0, mjd]
        ok = self._iskclimatology.UpdateCache(location)
        self._cache_has_been_called = ok
        return ok

    #------------------------------------------------------------------------------
    #           get_parameter
    #------------------------------------------------------------------------------

    @wrap_skif_functionfail
    def get_parameter(self, species: str, latitude: float, longitude: float, altitudes: np.ndarray, utc: float)->np.ndarray:
        """
        Get a profile from the climatology. The code will update the internal cache if necessary. You must make all the
        necessary calls to  `add_species` before the first call to `get_parameter`. Requesting to add  species after the first call to
        `update_cache` or  `get_parameter` is not supported.

        Parameters
        ----------
        species : str
            Species identifier of the species to query. Corresponds to :py:attr:`sasktran.Species.species`. The species
            must have been requested by a previous call to `load_species`
        latitude : float
            Latitude in degrees.
        longitude : float
            Longitude in degrees.
        altitudes : np.ndarray
            Profile altitude grid in meters.
        mjd : float
            A scalar UTC that can be converted to Modified julian date. It can be a datetime, datetime64 or (mjd) number

        Returns
        -------
        np.ndarray
            Climatology values at the specified altitudes.
        """

        mjd = arg_time.ut_to_mjd(utc)
        location = [latitude, longitude, 0, mjd]
        if (not self._cache_has_been_called): self._iskclimatology.UpdateCache(location)
        return self._iskclimatology.GetHeightProfile(species, location, np.asarray(altitudes))[1]

    #------------------------------------------------------------------------------
    #           add_species
    #------------------------------------------------------------------------------
    @wrap_skif_functionfail
    def add_species(self, speciesname):
        """
        Instructs the GEOS-CHEM climatology to load the requested variable when it next loads the internal cache. The
        function may be called multiple times to request multiple variables be loaded as long as all the calls are made before
        the first call to `update_cache` or `get_parameter`.

        Parameters
        ----------
        speciesname : str
            The name of any species supported by GEOS-CHEM. Complete list of species can be found in method `supportedspecies`
            of in the GEOS-CHEM documentation.

        Returns
        -------
        bool
            Returns True if successful.
        """

        return self._iskclimatology.SetProperty('add_species',speciesname)

    @wrap_skif_functionfail
    def force_year_to_use(self, year):
        """
        Forces GEOS-CHEM to load data files from the specified year regardless of the actual year requested by the user calculations.
        This is useful to allow the code to use GEOS CHEM data from one year even when performing calculations for
        another year. The `year` parameter actually runs from July of the given year to June of the next year, so
        specifying 2013 will force GEOS CHEM to load files from July of 2013 to June of 2014 regardless of the year entered by the user. Only
        the year field is modified and it is only applied to file I/O operations.

        Parameters
        ----------
        year : int
            The year of the files which will be loaded from file. The year runs from July of the specified year to June of the next year

        Returns
        -------
        bool
            Returns True if successful.
        """
        floatyear = float(year)
        self._iskclimatology.SetProperty('force_year_to_use',floatyear)

